@@ -16,7 +16,7 @@
 #
 #  partly based on work by Nokia Solutions and Networks Oyj
 """A parallel executor for Robot Framework test cases.
-Version 1.10.0
+Version 1.12
 
 Supports all Robot Framework command line options and also following
 options (these must be before normal RF options):
@@ -77,6 +77,7 @@ import datetime
 import uuid
 import random
 import traceback
+import socket
 from glob import glob
 from io import BytesIO, StringIO
 from collections import namedtuple
@@ -1259,10 +1260,18 @@ def _copy_output_artifacts(options, file_extensions=None, include_subfolders=Fal
 
 
 def _report_results(outs_dir, pabot_args, options, start_time_string, tests_root_name):
-    stats = {
-        "critical": {"total": 0, "passed": 0, "failed": 0},
-        "all": {"total": 0, "passed": 0, "failed": 0},
-    }
+    if ROBOT_VERSION < "4.0":
+        stats = {
+            "critical": {"total": 0, "passed": 0, "failed": 0},
+            "all": {"total": 0, "passed": 0, "failed": 0},
+        }
+    else:
+        stats = {
+            "total": 0,
+            "passed": 0,
+            "failed": 0,
+            "skipped": 0,
+        }
     if pabot_args["argumentfiles"]:
         outputs = []  # type: List[str]
         for index, _ in pabot_args["argumentfiles"]:
@@ -1290,16 +1299,22 @@ def _report_results(outs_dir, pabot_args, options, start_time_string, tests_root
 
 
 def _write_stats(stats):
-    crit = stats["critical"]
-    al = stats["all"]
-    _write(
-        "%d critical tests, %d passed, %d failed"
-        % (crit["total"], crit["passed"], crit["failed"])
-    )
-    _write(
-        "%d tests total, %d passed, %d failed"
-        % (al["total"], al["passed"], al["failed"])
-    )
+    if ROBOT_VERSION < "4.0":
+        crit = stats["critical"]
+        al = stats["all"]
+        _write(
+            "%d critical tests, %d passed, %d failed"
+            % (crit["total"], crit["passed"], crit["failed"])
+        )
+        _write(
+            "%d tests total, %d passed, %d failed"
+            % (al["total"], al["passed"], al["failed"])
+        )
+    else:
+        _write(
+            "%d tests, %d passed, %d failed, %d skipped."
+            % (stats["total"], stats["passed"], stats["failed"], stats["skipped"])
+        )
     _write("===================================================")
 
 
@@ -1356,12 +1371,18 @@ def _merge_one_run(
 
 def _update_stats(result, stats):
     s = result.statistics
-    stats["critical"]["total"] += s.total.critical.total
-    stats["critical"]["passed"] += s.total.critical.passed
-    stats["critical"]["failed"] += s.total.critical.failed
-    stats["all"]["total"] += s.total.all.total
-    stats["all"]["passed"] += s.total.all.passed
-    stats["all"]["failed"] += s.total.all.failed
+    if ROBOT_VERSION < "4.0":
+        stats["critical"]["total"] += s.total.critical.total
+        stats["critical"]["passed"] += s.total.critical.passed
+        stats["critical"]["failed"] += s.total.critical.failed
+        stats["all"]["total"] += s.total.all.total
+        stats["all"]["passed"] += s.total.all.passed
+        stats["all"]["failed"] += s.total.all.failed
+    else:
+        stats["total"] += s.total.total
+        stats["passed"] += s.total.passed
+        stats["failed"] += s.total.failed
+        stats["skipped"] +=  s.total.skipped
 
 
 # This is from https://github.com/django/django/blob/master/django/utils/glob.py
@@ -1412,9 +1433,20 @@ def _stop_message_writer():
     MESSAGE_QUEUE.join()
 
 
+def _get_free_port(pabot_args):
+    # Determines a free port using sockets.
+    free_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    free_socket.bind((pabot_args['pabotlibhost'], pabot_args['pabotlibport']))
+    free_socket.listen(5)
+    port = free_socket.getsockname()[1]
+    free_socket.close()
+    return port
+
+
 def _start_remote_library(pabot_args):  # type: (dict) -> Optional[subprocess.Popen]
     global _PABOTLIBURI
-    _PABOTLIBURI = "%s:%s" % (pabot_args["pabotlibhost"], pabot_args["pabotlibport"])
+    free_port = _get_free_port(pabot_args)
+    _PABOTLIBURI = "%s:%s" % (pabot_args["pabotlibhost"], free_port)
     if not pabot_args["pabotlib"]:
         return None
     if pabot_args.get("resourcefile") and not os.path.exists(
@@ -1432,7 +1464,7 @@ def _start_remote_library(pabot_args):  # type: (dict) -> Optional[subprocess.Po
             pabotlibname=pabotlib.__name__,
             resourcefile=pabot_args.get("resourcefile"),
             pabotlibhost=pabot_args["pabotlibhost"],
-            pabotlibport=pabot_args["pabotlibport"],
+            pabotlibport=free_port,
         ),
         shell=True,
     )
